<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2004-2019 The Cacti Group                                 |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/
include('./include/cli_check.php');
include_once($config['base_path'] . '/lib/poller.php');
include_once($config['base_path'] . '/plugins/wmitrack/wmitrack.php');

ini_set('memory_limit', '-1');

global $debug, $web, $track_errors;


$currentCycle=read_config_option('wmitrack_cycle');
if($currentCycle==11){
	$currentCycle=0;
}else{
	$currentCycle++;
}
set_config_option('wmitrack_cycle',$currentCycle);

//Get the devices we're going to attempt to log
$devices=db_fetch_assoc('SELECT * FROM cacti.wmi_track_hosts where disable=0 and cycle='.$currentCycle.' group by hostname');
$totalDevices=count($devices);
$stringDevice='devices';
if($totalDevices==1){
	$stringDevice='device';
}

cacti_log('NOTE: WMI Track (Cycle '.$currentCycle.') attempting to query '.$totalDevices.' '.$stringDevice);

$wmi  = new COM('WbemScripting.SWbemLocator');

$keyDevices=array();

foreach($devices as $device){
	
	$ok=true;
	$data=null;
	
	try{
		$wmic = $wmi->ConnectServer($device['hostname'], 'root\\cimv2', read_config_option('wmitrack_username'), read_config_option('wmitrack_password'));
		$wmic->Security_->ImpersonationLevel = 3;
		//Disks
		$dataDisk = $wmic->ExecQuery("select * from  Win32_LogicalDisk");
		//System
		$dataSystem = $wmic->ExecQuery("select * from  Win32_ComputerSystem");
		//Operating System
		$dataOperatingSystem = $wmic->ExecQuery("select * from  Win32_OperatingSystem");
		//Event Viewer (system)
		$dataEventViewer = $wmic->ExecQuery("Select * from Win32_NTLogEvent where LogFile='System' and Type='Error'");
	}catch(Exception $e){
		cacti_log('ERROR: WMI Track could not connect to '.$device['hostname']);
		$ok=false;
		//TODO: add a flag in DB for online
	}
	
	if($ok){
		//Read through system information
		if(sizeof($dataSystem)){
			//Clear data on this device
			db_execute("delete from cacti.wmi_track_data where device=".$device['id']);
			//System
			foreach($dataSystem as $wmiCall){
				//Name
				if(strlen($wmiCall->Name)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'Name','',$wmiCall->Name));
				}
				//System Type
				if(strlen($wmiCall->SystemType)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'SystemType','',$wmiCall->SystemType));
				}
				//Model
				if(strlen($wmiCall->Model)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'Model','',$wmiCall->Model));
				}
				//Status
				if(strlen($wmiCall->Status)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'Status','',$wmiCall->Status));
				}
			}
		}
		//Read through disk information
		if(sizeof($dataDisk)){
			//Disks
			foreach($dataDisk as $wmiCall){
				//Free space
				if(strlen($wmiCall->Name)>0 && strlen($wmiCall->FreeSpace)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'HDDFree',$wmiCall->Name,$wmiCall->FreeSpace));
				}
				//Total space
				if(strlen($wmiCall->Name)>0 && strlen($wmiCall->Size)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'HDDSize',$wmiCall->Name,$wmiCall->Size));
				}
			}
		}
		//Read through Operating System information
		if(sizeof($dataOperatingSystem)){
			//Operating System
			foreach($dataOperatingSystem as $wmiCall){
				//Caption
				if(strlen($wmiCall->Caption)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'OSName','',$wmiCall->Caption));
				}
				//Architecture
				if(strlen($wmiCall->OSArchitecture)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'OSArchitecture','',$wmiCall->OSArchitecture));
				}
				//Status
				if(strlen($wmiCall->Status)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'OSStatus','',$wmiCall->Status));
				}
				//Version
				if(strlen($wmiCall->Version)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'OSVersion','',$wmiCall->Version));
				}
			}
		}
		//Read through event viewer
		if(sizeof($dataEventViewer)){
			//Operating System
			foreach($dataEventViewer as $wmiCall){
				//System Error
				if(strlen($wmiCall->Message)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'SystemErrorMessage','',$wmiCall->Message));
				}
				//Time
				if(strlen($wmiCall->Message)>0){
					db_execute_prepared('INSERT INTO `cacti`.`wmi_track_data` (`device`,`object`,`oid`,`result`) VALUES(?,?,?,?)',array($device['id'],'SystemErrorTime','',$wmiCall->TimeGenerated));
				}
				break;
			}
		}
	}
	
}


//Get all devices we're already tracking
$devices=db_fetch_assoc('SELECT * FROM cacti.wmi_track_hosts group by hostname');
foreach($devices as $device){
	$keyDevices[strtolower($device['hostname'])]=1;
}

//Check for new devices
$submittedDevices=read_config_option('wmitrack_hosts');
$submittedDevices=explode(",",$submittedDevices);
foreach($submittedDevices as $device){
	$hostname=strtolower(trim($device));
	//Does this hostname already exist in track_hosts table
	if(isset($keyDevices[$hostname])){
		//Yes - mark it as seen
		$keyDevices[$hostname]=0;
	}else{
		//No - add it to the table
		db_execute_prepared('INSERT INTO `cacti`.`wmi_track_hosts` (`hostname`,`cycle`) VALUES(?,?)',array(trim($device),rand(0,12)));
		cacti_log('NOTE: WMI Track added '.$device.' to tracking table');
		$keyDevices[$hostname]=0;
	}
}
//Check for removed devices
foreach($keyDevices as $hostname=>$remove){
	if($remove==1){
		db_execute_prepared('delete from `cacti`.`wmi_track_hosts` where hostname=?',array(trim($hostname)));
		cacti_log('NOTE: WMI Track removed '.$hostname.' from tracking table');
	}
}
//Tidy redundant data
db_execute('delete `wmi_track_data` from `cacti`.`wmi_track_data` left join `cacti`.`wmi_track_hosts` on `wmi_track_data`.`device`=`wmi_track_hosts`.`id`  where `hostname` is null');
		

?>