<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2004-2019 The Cacti Group                                 |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/
function plugin_wmitrack_install(){
	global $config;
	api_plugin_register_hook('wmitrack', 'top_header_tabs',       'wmitrack_show_tab',             'setup.php');
	api_plugin_register_hook('wmitrack', 'top_graph_header_tabs', 'wmitrack_show_tab',             'setup.php');
	api_plugin_register_hook('wmitrack', 'config_settings', 'wmitrack_config_settings', 'setup.php');
	api_plugin_register_hook('wmitrack', 'config_arrays',         'wmitrack_config_arrays',        'setup.php');
	api_plugin_register_hook('wmitrack', 'poller_bottom', 'wmitrack_poller', 'setup.php');
	api_plugin_register_hook('wmitrack', 'page_head',             'wmitrack_page_head',            'setup.php');

	api_plugin_register_realm('wmitrack', 'wmitrack.php,wmitrack_view.php', 'View WMI Track', 1);
	
	
	//Cycle 0 - 11
	set_config_option('wmitrack_cycle','0');
	
	include_once($config['base_path'] . '/plugins/wmitrack/include/database.php');
	
	WMITrackDB::setup();
}

function plugin_wmitrack_uninstall(){
	return true;
}


function plugin_wmitrack_version(){
	global $config;
	$info = parse_ini_file($config['base_path'] . '/plugins/wmitrack/INFO', true);
	return $info['info'];
}

function plugin_wmitrack_check_config(){
	return true;
}


function wmitrack_show_tab(){
	global $config,$user_auth_realm_filenames;
	echo '<a href="' . $config['url_path'] . 'plugins/wmitrack/wmitrack_view.php"><img src="' . $config['url_path'] . 'plugins/wmitrack/images/tab_wmitrack.png" alt="'.__('WMITrack','wmitrack').'"></a>';
}

function wmitrack_config_settings(){
	global $tabs, $settings;
	$tabs['wmitrack'] = __('WMI Tracking','wmitrack');
	$settings['wmitrack']=array(
		'wmitrack_username' => array(
			'friendly_name'=> __('Username','wmitrack'),
			'method'=>'textbox',
			'description'=>__('User account used when performing WMI Queries','wmitrack'),
			'max_length' => '255'
		),
		'wmitrack_password' => array(
			'friendly_name'=> __('Password','wmitrack'),
			'method'=>'textbox_password',
			'max_length' => '255'
		),
		'wmitrack_hosts' => array(
			'friendly_name'=> __('Target Host Names','wmitrack'),
			'description' => htmlspecialchars(__('List of host names separated by commas (,).', 'wmitrack')),
			'method'=>'textarea',
			'textarea_rows' => '5',
			'textarea_cols' => '80',
		)
	);
}

function wmitrack_poller(){
	global $config;

	$command_string = read_config_option('path_php_binary');
	$extra_args = '-q ' . $config['base_path'] . '/plugins/wmitrack/poller.php';
	exec_background($command_string, $extra_args);
}

function wmitrack_config_arrays(){
	global $menu, $menu_glyphs, $config, $rows_selector;
	
}

function wmitrack_page_head(){
	global $config;
	echo "<script type='text/javascript' src='" . $config['url_path'] . "plugins/wmitrack/include/tabulator/js/tabulator.min.js'></script>\n";
	echo "<link type='text/css' href='" . $config['url_path'] . "plugins/wmitrack/include/tabulator/css/tabulator.css' rel='stylesheet'>\n";
	echo "<script type='text/javascript' src='" . $config['url_path'] . "plugins/wmitrack/include/lib.js'></script>\n";
	echo "<link type='text/css' href='" . $config['url_path'] . "plugins/wmitrack/include/style.css' rel='stylesheet'>\n";
	echo "<script type='text/javascript' src='" . $config['url_path'] . "plugins/wmitrack/include/moment.min.js'></script>\n";
	
}

?>