<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2004-2019 The Cacti Group                                 |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/
$guest_account = true;
chdir('../../');
include('./include/auth.php');
include_once('./plugins/wmitrack/wmitrack.php');

$title = __('WMI Tracking - View', 'wmitrack');

function wmitrack_get(){
	global $config;
	
	$dataWMI=WMITrack::getRecords();
	$keyData=array();
	foreach($dataWMI as $id=>$row){
			$name=(isset($row['Name']) ? $row['Name']:' - ');
			$systemType=(isset($row['SystemType']) ? $row['SystemType']:' - ');
			$model=(isset($row['Model']) ? $row['Model']:' - ');
			$sysStatus=(isset($row['Status']) ? $row['Status']:' - ');
			$osName=(isset($row['OSName']) ? $row['OSName']:' - ');
			$osVersion=(isset($row['OSVersion']) ? $row['OSVersion']:' - ');
			$osArchitecture=(isset($row['OSArchitecture']) ? $row['OSArchitecture']:' - ');
			$osStatus=(isset($row['OSStatus']) ? $row['OSStatus']:' - ');
			
			$lowestFree=-1;
			$stringHDDs="";
			foreach($row['HDDFree'] as $oid=>$value){
				$bufferFree=round(($value/1000000000),2);
				$stringHDDs.="<strong>".$oid."</strong> ".$bufferFree."<Br>";
				if($lowestFree==-1 || $bufferFree<$lowestFree){
					$lowestFree=$bufferFree;
				}
			}
			
			$lastError=(isset($row['SystemErrorTime']) ? date('d/m/Y H:i',strtotime(substr($row['SystemErrorTime'],0,12))): " - ");
			$lastErrorMessage=(isset($row['SystemErrorMessage']) ? $row['SystemErrorMessage']:' - ');
			
			$keyData[]=array('name'=>$name,'systemType'=>$systemType,'model'=>$model,'sysStatus'=>$sysStatus,'osName'=>$osName,'osVersion'=>$osVersion,'osArchitecture'=>$osArchitecture,'osStatus'=>$osStatus,'hdds'=>$stringHDDs,'lowestHDD'=>$lowestFree,'lastError'=>$lastError,'lastErrorMessage'=>$lastErrorMessage);
	}
	echo json_encode($keyData);
}

function wmitrack_view() {
	global $title, $config, $item_rows;
	
	$dataWMI=WMITrack::getRecords();
	
	
	html_start_box('','100%','','3','center','');
	
	?>
	<div id=tableWMI></div>
	<script type='text/javascript'>
	
	var table = new Tabulator("#tableWMI", {
		//data:tabledata,           //load row data from array
		layout:"fitColumns",      //fit columns to width of table
		resizableRows:true,       //allow row order to be changed
		initialSort:[             //set the initial sort order of the data
			{column:"name", dir:"asc"},
		],
		columns:[                 //define the table columns
			{title:"Device", field:"name",headerFilter:"select",headerFilterParams:{values:true}},
			{title:"Platform", field:"systemType", align:"left",headerFilter:"select",headerFilterParams:{values:true}},
			{title:"Model", field:"model",headerFilter:"select",headerFilterParams:{values:true}},
			{title:"Sys Status", field:"sysStatus",headerFilter:"select",headerFilterParams:{values:true}},
			{title:"OS", field:"osName",headerFilter:"select",headerFilterParams:{values:true}},
			{title:"Version", field:"osVersion",headerFilter:"select",headerFilterParams:{values:true}},
			{title:"Architecture", field:"osArchitecture",headerFilter:"select",headerFilterParams:{values:true}},
			{title:"Status", field:"osStatus",headerFilter:"select",headerFilterParams:{values:true}},
			{title:"HDDs",formatter:'html',headerSort:false, field:"hdds"},
			{title:"Lowest HDD", field:"lowestHDD"},
			{title:"System Event Error", field:"lastError",sorter:"date"}
		],
		rowClick:function(e, row){
			alert("Last Error Message: " + row.getData().lastErrorMessage)
		}
	});
	
	function populate(data){
		table.replaceData(data);
	}
	
	function getData(){
		let xhr = new XMLHttpRequest();
		xhr.open('GET', '<?php echo $config['url_path'] . "plugins/wmitrack/wmitrack_view.php?json=1"; ?>');
		xhr.send();
		xhr.onload = function() {
			if (xhr.status == 200) { // analyze HTTP status of the response
				populate(JSON.parse(xhr.response));
			}
		}
	}
	getData();
	window.setInterval(function(){
		getData();
	}, 60000);
	</script>
	<?php
	
	
}

if(!isset($_GET['json'])){
	general_header();
	wmitrack_view();
	bottom_footer();
}else{
	wmitrack_get();
}

?>